
using System.Text;
using System.Diagnostics;
using System.Threading ;
using System.Collections.Generic;

using Microsoft.AspNetCore.Authentication.Cookies;
using Microsoft.AspNetCore.Http.Features;
using Microsoft.AspNetCore.Server.Kestrel.Core;
using Microsoft.IdentityModel.Tokens;
using Microsoft.OpenApi.Models;

using HttpClientStudy.Model;
using HttpClientStudy.Service;
using HttpClientStudy.Config;

namespace HttpClientStudy.WebApp
{
    /// <summary>
    /// HttpClientѧϰWebAPIĿ
    /// </summary>
    public class Program
    {
        /// <summary>
        /// Main
        /// </summary>
        /// <param name="args"></param>
        public static void Main(string[] args)
        {
            string projectAndMutexName = WebApiConfigManager.GetWebApiConfigOption().CurrentValue.WebAppMutexName;
            var mutex = new Mutex(true,projectAndMutexName,out bool createdResult);
            if (createdResult == false)
            {
                Console.WriteLine($"ѾһʵУʵ˳");
                return;
            }

            var builder = WebApplication.CreateBuilder(args);

            #region ע
            builder.Services.AddControllers();
            builder.Services.AddEndpointsApiExplorer();

            //Sessionм
            builder.Services.AddDistributedMemoryCache();

            //Session
            builder.Services.AddSession(option =>
            {
                option.Cookie.Name = "HttpClientStudy";
                option.IOTimeout = TimeSpan.FromHours(1);
                option.IdleTimeout = TimeSpan.FromHours(1);
            });

            //Kestrelѡ
            builder.Services.Configure<KestrelServerOptions>(option =>
            {
                //ASP.NET Core 3.0 ֮ǰİ汾AllowSynchronousIO Ĭǿ
                // true ͬ IO ,Getе.ֱֻӴжȡԶģͰ󶨡
                option.AllowSynchronousIO = true;
            });

            //Formύѡ
            builder.Services.Configure<FormOptions>(options =>
            {
                //options.BufferBody = true;
                options.MultipartBodyLengthLimit = long.MaxValue;
                options.MultipartBoundaryLengthLimit = int.MaxValue;
                options.MultipartHeadersCountLimit = int.MaxValue;
                options.MultipartHeadersLengthLimit = int.MaxValue;
            });

            //Swagger
            builder.Services.AddSwaggerGen(setup =>
            {
                #region Swaggerĵ
                //nameΪSwaggerUISwaggerEndpointе{documentName}
                //߱뱣һ£쳣
                setup.SwaggerDoc(name: "v1", new Microsoft.OpenApi.Models.OpenApiInfo { Title = "HttpClientѧϰ", Version = "1" });
                #endregion

                #region xmlע
                var xmlCommentFiles = System.IO.Directory.GetFiles(AppDomain.CurrentDomain.BaseDirectory, "HttpClientStudy.*.xml", System.IO.SearchOption.TopDirectoryOnly);
                foreach (var xmlFile in xmlCommentFiles)
                {
                    //includeControllerXmlCommentsǷÿϵxmlע
                    setup.IncludeXmlComments(filePath: xmlFile, includeControllerXmlComments: true);

                    setup.UseInlineDefinitionsForEnums();
                }
                #endregion

                #region ýӿAuthȨť
                setup.AddSecurityDefinition("Bearer", new OpenApiSecurityScheme
                {
                    Description = "BearerTokenBearer {Token}",

                    //jwtĬϵĲ
                    Name = "Authorization",

                    //jwtĬϴ Authorization Ϣλã˴Ϊͷ
                    In = ParameterLocation.Header,

                    //֤ͣ˴ʹApi Key
                    Type = SecuritySchemeType.ApiKey
                });
                #endregion

                #region ָӦ÷Χ
                setup.AddSecurityRequirement(new OpenApiSecurityRequirement
                {
                    {
                        new OpenApiSecurityScheme
                        {
                            Reference = new OpenApiReference
                            {
                                Id = "Bearer",
                                Type = ReferenceType.SecurityScheme
                            }
                        },
                        new List<string>()
                    }
                });
                #endregion

                //ע
                setup.EnableAnnotations();
            });

            //CORS
            builder.Services.AddCors(option =>
            {
                option.AddPolicy("AllowAll", builder =>
                {
                    builder.SetIsOriginAllowed(_ => true).AllowAnyMethod().AllowAnyHeader().AllowCredentials();
                });
            });

            //֤
            builder.Services //֤ܹ
                 .AddAuthentication(authOption =>
                 {
                     authOption.DefaultAuthenticateScheme = CookieAuthenticationDefaults.AuthenticationScheme;
                     authOption.DefaultChallengeScheme = CookieAuthenticationDefaults.AuthenticationScheme;
                     authOption.DefaultAuthenticateScheme = CookieAuthenticationDefaults.AuthenticationScheme;
                     authOption.DefaultSignOutScheme = CookieAuthenticationDefaults.AuthenticationScheme;
                     authOption.DefaultSignInScheme = CookieAuthenticationDefaults.AuthenticationScheme;
                 })
                 //Cookie֤
                 .AddCookie(CookieAuthenticationDefaults.AuthenticationScheme, option =>
                {
                    option.Cookie.Name = ".eds.editor.cookie.authentication.oa2";//ô洢û¼ϢûTokenϢCookie
                    option.Cookie.HttpOnly = true;//ô洢û¼ϢûTokenϢCookie޷ͨͻű(JavaScript)ʵ
                    option.ExpireTimeSpan = TimeSpan.FromDays(3);// ʱ
                    option.SlidingExpiration = true;// ǷڹʱʱԶ
                    option.LoginPath = "/Account/Login";
                    option.LogoutPath = "/Account/LoginOut";
                    //option.AccessDeniedPath = "/Account/Login";
                })
                 //֤
                 .AddJwtBearer(option =>
                {
                    option.TokenValidationParameters = new TokenValidationParameters
                    {
                        ValidIssuer = "WWW.WANGGAOFENG.CN",
                        ValidAudience = "WWW.WANGGAOFENG.CN",
                        ValidateIssuer = true,
                        ValidateLifetime = true,
                        IssuerSigningKey = new SymmetricSecurityKey(Encoding.UTF8.GetBytes("0123456789abcdefghigklmnopqrstdf41sadfweqtdfghsdfgsdfweqr")),

                        //ʱ䣬ܵЧʱʱjwtĹʱ
                        ClockSkew = TimeSpan.FromSeconds(0)
                    };
                });

            //Ȩ
            builder.Services.AddAuthorization();

            //
            builder.Services.AddHealthChecks();

            //ͨ

            builder.Services.AddScoped(provider => new Account() { Id = 0, Name = "עʾ", Password = "123456", Role = "IoC" });
            builder.Services.AddScoped<AccountService>();
            #endregion

            var app = builder.Build();

            #region Httpܵ

            //ʱͳм
            app.UseMiddleware<UsedTimeMiddleware>();

            app.MapHealthChecks("api/health");

            app.UseSwagger();
            app.UseSwaggerUI(setup =>
            {
                setup.EnableDeepLinking();
                setup.DisplayRequestDuration();
                setup.ShowCommonExtensions();
                setup.ShowExtensions();
                setup.EnableFilter();
            });
            app.UseCors("AllowAll");
            app.UseAuthorization();

            app.MapControllers();

            app.UseAuthorization();
            #endregion

            app.Run();
        }
    }
}
